
function [P_ci, PHI_ci, RMSRm_ci, RMSRc_ci, RMSRcm_ci, RMSRtot_ci] = BootStrap_acquiobuls (X,R,IRTModel,m_aq,m_co,rev,n_boot,CIlevel,disp)
 
% X -> matriz de datos con marcadores sin revertir
% R -> matriz de correlacion interitems con marcadores sin revertir
% IRTModel : 0 Lineal; 1: binario; 2: politomico
% nm -> numero de marcadores (las primeras nm filas)
% nc -> numero de items de contenido
% N  -> numero de participantes
% rev -> lista de items que han sido revertidos (reverse-scored)

[n,j]=size(X);

total = n_boot + n;
ch=0;

% Sample outcomes
[P, PHI, RMSRm, RMSRc, RMSRcm, RMSRtot]=acquiobuls(R,m_aq,m_co,n,rev);

[m,q]=size(P);

% Simple Bootstrap
P_b=zeros(j*q,n_boot);
PHI_b=zeros(1,n_boot);
RMSRm_b=zeros(1,n_boot);
RMSRc_b=zeros(1,n_boot);
RMSRcm_b=zeros(1,n_boot);
RMSRtot_b=zeros(1,n_boot);

if disp == 1,
   tstart = tic;
   tmp3 = 0; bar1 = waitbar(tmp3,'Accelerated bootstrap sampling');
end;    

for i=1:n_boot
  again=1;
  while again,
     b=ceil(rand(n,1)*n);  % bootstrap sample
     X_bi=X(b,:);
     if min(std(X_bi)') > 0.001,
         again = 0;
     end;
  end;
  if IRTModel == 1,
     R_bi=corrcoef(X_bi);
  else,
      R_bi=polychomap(X_bi,min(X_bi),max(X_bi),3,0,0);
        %NPD check
        D=eig(R_bi);
        if min(D) <0,
            %NONLINEAR TRANSFORMATION
            R_bi = sr(R_bi,n);
        end;
      
  end;
  
  [P_bi, PHI_bi, RMSRm_bi, RMSRc_bi, RMSRcm_bi, RMSRtot_bi]=acquiobuls(R_bi,m_aq,m_co,n,rev);
  
  % collect bootstrap solutions
  P_b(:,i)=P_bi(:);
  PHI_b(i)=PHI_bi(1,2);
  RMSRm_b(i)=RMSRm_bi;
  RMSRc_b(i)=RMSRc_bi;
  RMSRcm_b(i)=RMSRcm_bi;
  RMSRtot_b(i)=RMSRtot_bi;

  ch = ch + 1;
  if (disp >0),
    compT = toc(tstart);
    compT = compT*(total-ch)/ch;
    s_tmp=['Accelerated bootstrap sampling (Remaining time = ' Time2String( compT ) ')'];
    waitbar(ch/total,bar1,s_tmp);
  end;

end;

% Expanded: dataset expanded to contain the i-th observation twice
P_ex=zeros(j*q,n);
PHI_ex=zeros(1,n);
RMSRm_ex=zeros(1,n);
RMSRc_ex=zeros(1,n);
RMSRcm_ex=zeros(1,n);
RMSRtot_ex=zeros(1,n);

for i=1:n
  Xi=[X;X(i,:)];     %dataset expanded to contain the i-th observation twice
  if IRTModel == 1,  
     R_i=corrcoef(Xi);
  else,
      R_i=polychomap(Xi,min(Xi),max(Xi),3,0,0);
        %NPD check
        D=eig(R_i);
        if min(D) <0,
            %NONLINEAR TRANSFORMATION
            R_i = sr(R_i,n+1);
        end;
  end;
  
  [P_i, PHI_i, RMSRm_i, RMSRc_i, RMSRcm_i, RMSRtot_i]=acquiobuls(R_i,m_aq,m_co,n+1,rev);
  
  % collect bootstrap solutions
  P_ex(:,i)=P_i(:);
  PHI_ex(i)=PHI_i(1,2);
  RMSRm_ex(i)=RMSRm_i;
  RMSRc_ex(i)=RMSRc_i;
  RMSRcm_ex(i)=RMSRcm_i;
  RMSRtot_ex(i)=RMSRtot_i;

  ch = ch + 1;
  if (disp >0),
    compT = toc(tstart);
    compT = compT*(total-ch)/ch;
    s_tmp=['Accelerated bootstrap sampling (Remaining time = ' Time2String( compT ) ')'];
    waitbar(ch/total,bar1,s_tmp);
  end;
  
end;

%computes Bias corrected CI
[P_ci]=BCa(P(:),P_b,P_ex,CIlevel);
[PHI_ci]=BCa(PHI(1,2),PHI_b,PHI_ex,CIlevel);
[RMSRm_ci]=BCa(RMSRm,RMSRm_b,RMSRm_ex,CIlevel);
[RMSRc_ci]=BCa(RMSRc,RMSRc_b,RMSRc_ex,CIlevel);
[RMSRcm_ci]=BCa(RMSRcm,RMSRcm_b,RMSRcm_ex,CIlevel);
[RMSRtot_ci]=BCa(RMSRtot,RMSRtot_b,RMSRtot_ex,CIlevel);

if disp == 1,close(bar1);end;

return;
